from rest_framework import viewsets
from .serializers import ServerSerializer
from .models import Server


class ServerViewSet(viewsets.ModelViewSet):
    """
    API endpoint that allows users to be viewed or edited.
    """
    queryset = Server.objects.all()
    serializer_class = ServerSerializer



import mock
import random
from django.test import TestCase
from game_info.models import Server, Player, Rule
from valve.source import messages, util, a2s


class MockServerQuerier():
    def __init__(self, address):
        pass

    def get_info(self):
        i = messages.InfoResponse()
        i['response_type'] = 73
        i['protocol'] = 17
        i['server_name'] = u'A test server'
        i['map'] = u'ctf_2fort'
        i['folder'] = u'tf'
        i['game'] = u'Team Fortress'
        i['app_id'] = 440
        i['player_count'] = 31
        i['max_players'] = 31
        i['bot_count'] = 0
        i['server_type'] = util.ServerType(100)
        i['platform'] = util.Platform(108)
        i['password_protected'] = 0
        i['vac_enabled'] = 1
        i['version'] = u'3200676'
        return i

    def get_players(self):
        players = messages.PlayersResponse()
        players['player_count'] = 32
        players['response_type'] = 68
        players['players'] = []
        for i in range(1, 33):
            player = messages.PlayerEntry()
            player['index'] = 0
            player['name'] = "Player %d" % (i)
            player['score'] = random.randrange(1000)
            player['duration'] = random.uniform(1, 10000)
            players['players'].append(player)
        return players

    def get_rules(self):
        r = messages.RulesResponse()
        r['rule_count'] = 2
        r['rules'] = {
            'sv_alltalk': u'0',
            'sv_voiceenable': u'1'
        }
        return r


class MockServerDownQuerier():
    def __init__(self, address):
        pass

    def get_info(self):
        raise a2s.NoResponseError("Timed out waiting for response")

    def get_players(self):
        raise a2s.NoResponseError("Timed out waiting for response")

    def get_rules(self):
        raise a2s.NoResponseError("Timed out waiting for response")


class ServerTest(TestCase):

    def create_server(self, title="Test Server", host="example.org", port=27015):
        return Server.objects.create(title=title, host=host, port=port)

    def create_player(self, server, name="Bob", score=100, duration=1000.0):
        return Player.objects.create(server=server, name=name, score=score, duration=duration)

    def create_rule(self, server, cvar="sv_gravity", value="800"):
        return Rule.objects.create(server=server, cvar=cvar, value=value)

    def test_server_creation(self):
        s = self.create_server()
        self.assertTrue(isinstance(s, Server))
        self.assertEqual(s.__unicode__(), s.title)

    @mock.patch('valve.source.a2s.ServerQuerier', MockServerQuerier)
    def test_update_info(self):
        s = self.create_server()
        self.assertTrue(isinstance(s, Server))
        self.assertEqual(s.update_info(), True)

    @mock.patch('valve.source.a2s.ServerQuerier', MockServerQuerier)
    def test_update_players(self):
        s = self.create_server()
        self.assertTrue(isinstance(s, Server))
        self.assertEqual(s.update_players(), True)

    @mock.patch('valve.source.a2s.ServerQuerier', MockServerQuerier)
    def test_update_rules(self):
        s = self.create_server()
        self.assertTrue(isinstance(s, Server))
        self.assertEqual(s.update_rules(), True)

    @mock.patch('valve.source.a2s.ServerQuerier', MockServerDownQuerier)
    def test_update_info_exception(self):
        s = self.create_server()
        self.assertTrue(isinstance(s, Server))
        self.assertEqual(s.update_info(), False)

    @mock.patch('valve.source.a2s.ServerQuerier', MockServerDownQuerier)
    def test_update_players_exception(self):
        s = self.create_server()
        self.assertTrue(isinstance(s, Server))
        self.assertEqual(s.update_players(), False)

    @mock.patch('valve.source.a2s.ServerQuerier', MockServerDownQuerier)
    def test_update_rules_exception(self):
        s = self.create_server()
        self.assertTrue(isinstance(s, Server))
        self.assertEqual(s.update_rules(), False)

    def test_player(self):
        s = self.create_server()
        p = self.create_player(s)
        self.assertTrue(isinstance(p, Player))
        self.assertEqual(p.__unicode__(), p.name)

    def test_rule(self):
        s = self.create_server()
        r = self.create_rule(s)
        self.assertTrue(isinstance(r, Rule))
        self.assertEqual(r.__unicode__(), r.cvar)



from django.test import TestCase
from django.contrib.admin.sites import AdminSite

from game_info.admin import ServerAdmin
from game_info.models import Server


class AdminTest(TestCase):

    def test_server_admin(self):
        server_admin = ServerAdmin(Server, AdminSite())




from django.db import models
import valve.source.a2s

SERVER_TYPES = (
    (68, 'Dedicated'),
    (100, 'Dedicated'),
    (108, 'Non-dedicated'),
    (112, 'SourceTV'),
)

PLATFORMS = (
    (76, 'Linux'),
    (108, 'Linux'),
    (109, 'Mac OS X'),
    (111, 'Mac OS X'),
    (119, 'Windows')
)


class Server(models.Model):
    title = models.CharField(max_length=200)
    host = models.CharField(max_length=200)
    port = models.IntegerField()

    # Whether we should run a2s_info queries.
    get_info = models.BooleanField(default=True)

    # Whether we should get a list of players.
    get_players = models.BooleanField(default=True)

    # Whether we should get the servers rules (cvars).
    get_rules = models.BooleanField(default=True)

    # True if the server is contactable, otherwise False.
    up = models.BooleanField(default=True, editable=False)

    updated_at = models.DateTimeField(auto_now=True)

    def update_info(self):
        a2s = valve.source.a2s.ServerQuerier((self.host, self.port))
        try:
            info = a2s.get_info()
            self.up = True
            self.save()
        except valve.source.a2s.NoResponseError:
            self.up = False
            self.save()
            return False
        info_model = Info(server=self)
        info_model.server_name = info['server_name']
        info_model.map = info['map']
        info_model.folder = info['folder']
        info_model.game = info['game']
        info_model.app_id = info['app_id']
        info_model.player_count = info['player_count']
        info_model.max_players = info['max_players']
        info_model.bot_count = info['bot_count']
        info_model.server_type = info['server_type'].value
        info_model.platform = info['platform'].value
        info_model.password_protected = info['password_protected']
        info_model.vac_enabled = info['vac_enabled']
        info_model.version = info['version']
        info_model.save()
        return True

    def update_players(self):
        a2s = valve.source.a2s.ServerQuerier((self.host, self.port))
        try:
            players = a2s.get_players()
            self.up = True
            self.save()
        except valve.source.a2s.NoResponseError:
            self.up = False
            self.save()
            return False

        player_models = []
        for player in players['players']:
            player_models.append(Player(
                server=self,
                name=player['name'],
                score=player['score'],
                duration=player['duration']
            ))
        self.player_set.all().delete()
        Player.objects.bulk_create(player_models)
        return True

    def update_rules(self):
        a2s = valve.source.a2s.ServerQuerier((self.host, self.port))
        try:
            rules = a2s.get_rules()
            self.up = True
            self.save()
        except valve.source.a2s.NoResponseError:
            self.up = False
            self.save()
            return False

        rule_models = []
        for cvar, value in rules['rules'].items():
            rule_models.append(Rule(
                server=self,
                cvar=cvar,
                value=str(value)
            ))
        self.rule_set.all().delete()
        Rule.objects.bulk_create(rule_models)
        return True

    def __unicode__(self):
        return self.title


class Info(models.Model):
    """
        Stores a game servers response to a2s_info, this contains
        General information about the server, such as player count and
        the current map.
    """

    server = models.OneToOneField(
        Server,
        on_delete=models.CASCADE,
        primary_key=True,
    )

    # The name of the server as returned by a2s_info
    server_name = models.CharField(max_length=256)

    # The name of the map the server is currently running
    map = models.CharField(max_length=64)

    # The gamedir of the mod being ran by the server, E.g "tf" or "csgo"
    folder = models.CharField(max_length=64)

    # A string identifying the game being ran by the server
    game = models.CharField(max_length=64)

    # The numberic application ID of the game ran by the server, note that the
    # ID of the client is returned, not the server. For example TF2 is 440
    # instead of 232250 which is the server software
    app_id = models.IntegerField()

    player_count = models.IntegerField()
    max_players = models.IntegerField()
    bot_count = models.IntegerField()
    server_type = models.IntegerField(choices=SERVER_TYPES)
    platform = models.IntegerField(choices=PLATFORMS)
    password_protected = models.BooleanField()
    vac_enabled = models.BooleanField()
    version = models.IntegerField()


class Player(models.Model):
    """
        Stores a game servers response to a2s_player, this contains
        a list of who is playing, their score, etc.
    """
    server = models.ForeignKey(Server)
    name = models.CharField(max_length=64)
    score = models.IntegerField()
    duration = models.FloatField()

    def __unicode__(self):
        return self.name


class Rule(models.Model):
    """
        Stores a subset of a server's console variables (often referred to as
        'cvars',) specifically those which have the ``FCVAR_NOTIFY`` flag set
        on them. These cvars are used to indicate game mode's configuration,
        such as the gravity setting for the map or whether friendly fire is
        enabled or not. You could also use this to transmit data from the
        server to django by having your plugin create a cvar.
    """
    server = models.ForeignKey(Server)
    cvar = models.CharField(max_length=64)
    value = models.CharField(max_length=64)

    def __unicode__(self):
        return self.cvar



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Player',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=64)),
                ('score', models.IntegerField()),
                ('duration', models.FloatField()),
            ],
        ),
        migrations.CreateModel(
            name='Rule',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('cvar', models.CharField(max_length=64)),
                ('value', models.CharField(max_length=64)),
            ],
        ),
        migrations.CreateModel(
            name='Server',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=200)),
                ('host', models.CharField(max_length=200)),
                ('port', models.IntegerField()),
                ('get_info', models.BooleanField(default=True)),
                ('get_players', models.BooleanField(default=True)),
                ('get_rules', models.BooleanField(default=True)),
                ('up', models.BooleanField(default=True, editable=False)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='Info',
            fields=[
                ('server', models.OneToOneField(primary_key=True, serialize=False, to='game_info.Server')),
                ('server_name', models.CharField(max_length=256)),
                ('map', models.CharField(max_length=64)),
                ('folder', models.CharField(max_length=64)),
                ('game', models.CharField(max_length=64)),
                ('app_id', models.IntegerField()),
                ('player_count', models.IntegerField()),
                ('max_players', models.IntegerField()),
                ('bot_count', models.IntegerField()),
                ('server_type', models.IntegerField(choices=[(68, b'Dedicated'), (100, b'Dedicated'), (108, b'Non-dedicated'), (112, b'SourceTV')])),
                ('platform', models.IntegerField(choices=[(76, b'Linux'), (108, b'Linux'), (109, b'Mac OS X'), (111, b'Mac OS X'), (119, b'Windows')])),
                ('password_protected', models.BooleanField()),
                ('vac_enabled', models.BooleanField()),
                ('version', models.IntegerField()),
            ],
        ),
        migrations.AddField(
            model_name='rule',
            name='server',
            field=models.ForeignKey(to='game_info.Server'),
        ),
        migrations.AddField(
            model_name='player',
            name='server',
            field=models.ForeignKey(to='game_info.Server'),
        ),
    ]



from django.conf.urls import url, include
from rest_framework import routers
from .views import ServerViewSet

router = routers.DefaultRouter()
router.register(r'server', ServerViewSet)

# Wire up our API using automatic URL routing.
# Additionally, we include login URLs for the browsable API.
urlpatterns = [
    url(r'^', include(router.urls), name='game_info'),
]



from rest_framework import serializers
from .models import Server, Info, Rule, Player


class InfoSerializer(serializers.ModelSerializer):
    class Meta:
        model = Info
        exclude = ("server", )


class RuleSerializer(serializers.ModelSerializer):
    class Meta:
        model = Rule
        exclude = ("id", "server")


class PlayerSerializer(serializers.ModelSerializer):
    class Meta:
        model = Player
        exclude = ("id", "server")


class ServerSerializer(serializers.ModelSerializer):
    info = InfoSerializer(read_only=True)
    players = PlayerSerializer(source='player_set', many=True)
    rules = RuleSerializer(source='rule_set', many=True)

    class Meta:
        model = Server
        exclude = ("get_info", "get_players", "get_rules")



from django.test import TestCase
from django.core.urlresolvers import reverse
from game_info.views import ServerViewSet
from game_info.models import Server


class ViewTest(TestCase):
    def test_server_list_view(self):
        s = Server(title="Title", host="example.org", port=27015)
        s.save()
        url = reverse("server-list")
        resp = self.client.get(url)

        self.assertEqual(resp.status_code, 200)
        self.assertIn(s.title, resp.content)




from django.core.management.base import BaseCommand, CommandError
from game_info.models import Server, Info, Player, Rule


class Command(BaseCommand):
    args = ''
    help = 'Update the game server information'

    def handle(self, *args, **options):
        for server in Server.objects.all():
            if server.get_info:
                server.update_info()
            if server.get_players:
                server.update_players()
            if server.get_rules:
                server.update_rules()



from django.core.management import call_command
from django.test import TestCase
from game_info.models import Server


class CommandTest(TestCase):
    def create_server(self, title="Test Server", host="example.org", port=27015):
        return Server.objects.create(title=title, host=host, port=port)

    def test_update_game_info(self):
        self.create_server().save()
        call_command('update_game_info')
